from django.conf import settings
from django.contrib.auth.models import Group, Permission
from django.core.management.base import BaseCommand
from django.db.models import QuerySet


class Command(BaseCommand):
    help = "add permissions"

    def add_permissions(self, group: Group, permissions: QuerySet, name: str):
        self.stdout.write(f"Adding {name} permissions to the '{group.name}' group... ", ending="")
        self.stdout.flush()
        group.permissions.add(*permissions)
        self.stdout.write(self.style.SUCCESS(str(len(permissions))))

    def handle(self, *args, **options):
        privileged_group = Group.objects.get(name=settings.PRIVILEGED_GROUP)
        host_group = Group.objects.get(name=settings.ENTITLED_GROUPS[0])
        host_plus_group = Group.objects.get(name=settings.ENTITLED_GROUPS[1])

        app_permissions = Permission.objects.filter(content_type__app_label="program").exclude(
            codename__startswith="edit"
        )
        default_model_permissions = (
            Permission.objects.filter(content_type__model__in=["note", "notelink"])
            .exclude(codename__startswith="edit")
            .exclude(codename__startswith="create")
            .exclude(codename__startswith="update")
        )
        change_permissions = Permission.objects.filter(
            codename__startswith="change", content_type__model__in=["host", "note", "show"]
        )
        edit_permissions = Permission.objects.filter(
            codename__startswith="edit", content_type__model__in=["host", "note", "show"]
        )
        create_permissions = Permission.objects.filter(
            codename__startswith="create", content_type__model__in=["note"]
        )
        update_permissions = Permission.objects.filter(
            codename__startswith="update", content_type__model__in=["host", "note", "show"]
        )
        custom_add_permissions = Permission.objects.filter(
            codename__startswith="add__", content_type__model="playlist"
        )

        self.add_permissions(privileged_group, app_permissions, "default app level")
        self.add_permissions(privileged_group, edit_permissions, "custom edit field")
        self.add_permissions(privileged_group, create_permissions, "custom create")
        self.add_permissions(privileged_group, update_permissions, "custom update")
        self.add_permissions(privileged_group, custom_add_permissions, "custom add")
        self.add_permissions(host_group, default_model_permissions, "default model")
        self.add_permissions(host_plus_group, change_permissions, "default change")
        self.add_permissions(host_plus_group, edit_permissions, "custom edit field")
        self.add_permissions(host_plus_group, custom_add_permissions, "custom add")
