#
# steering, Programme/schedule management for AURA
#
# Copyright (C) 2011-2017, 2020, Ernesto Rico Schmidt
# Copyright (C) 2017-2019, Ingo Leindecker
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the Free
# Software Foundation, either version 3 of the License, or (at your option) any
# later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for more
# details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from oidc_provider.lib.claims import ScopeClaims

from django.conf import settings
from django.utils import timezone
from django.utils.translation import gettext as _
from program.models import Show


class AuraScopeClaims(ScopeClaims):
    info_username = (
        _("username"),
        _("Your username."),
    )

    def scope_username(self):
        return {
            "username": self.user.username,
            "privileged": self.user.is_superuser
            or settings.PRIVILEGED_GROUP in self.user.groups.values_list("name", flat=True),
            "entitled": any(
                [
                    name in settings.ENTITLED_GROUPS
                    for name in self.user.groups.values_list("name", flat=True)
                ]
            ),
        }

    info_aura_shows = (
        _("AURA Shows"),
        _("AURA shows you have access to."),
    )

    def scope_aura_shows(self):
        # we use annotated objects because filtering using properties is not possible
        owned_shows = Show.objects.with_max_timeslot_start().filter(owners=self.user)
        public_shows = Show.objects.with_max_timeslot_start().filter(is_public=True)

        if settings.FILTER_ACTIVE_SHOWS_USING["is_active"]:
            # use the `is_active` field
            owned_shows = owned_shows.filter(is_active=True)
            public_shows = public_shows.filter(is_active=True)

        if settings.FILTER_ACTIVE_SHOWS_USING["max_timeslot_start"]:
            # use the `max_timeslot_start` annotation
            now = timezone.now()
            owned_shows = owned_shows.filter(max_timeslot_start__gt=now)
            public_shows = public_shows.filter(max_timeslot_start__gt=now)

        return {
            "ownedShowIds": list(owned_shows.values_list("id", flat=True)),
            "publicShowIds": list(public_shows.values_list("id", flat=True)),
        }
